@@ -10,7 +10,7 @@ use POSIX qw(locale_h);
 setlocale(LC_NUMERIC,"C");
 
 
-$VERSION = '23.2';
+$VERSION = '24.1';
 
 # Some function might be excluded from export and assessment.
 our @EXCLUDED_FUNCTION = ('SQUIRREL_GET_ERROR_OFFSET');
@@ -338,7 +338,9 @@ sub _column_comments
 
 	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	my %data = ();
-	while (my $row = $sth->fetch) {
+	while (my $row = $sth->fetch)
+	{
+		next if (!$self->is_in_struct($row->[2], $row->[0]));
 		$data{$row->[2]}{$row->[0]} = $row->[1];
 	}
 	return %data;
@@ -346,7 +348,7 @@ sub _column_comments
 
 sub _column_info
 {
-	my ($self, $table, $owner, $objtype, $recurs) = @_;
+	my ($self, $table, $owner, $objtype, $recurs, @expanded_views) = @_;
 
 	$objtype ||= 'TABLE';
 
@@ -387,9 +389,8 @@ sub _column_info
 FROM INFORMATION_SCHEMA.COLUMNS
 $condition
 ORDER BY ORDINAL_POSITION};
-	# Version below 5.5 do not have DATA_TYPE column it is named DTD_IDENTIFIER
-	if ($self->{db_version} < '5.5.0') {
-		$str =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
+	if ($self->{db_version} < '5.7.0') {
+		$str =~ s/, GENERATION_EXPRESSION//;
 	}
 	my $sth = $self->{dbh}->prepare($str);
 	if (!$sth) {
@@ -404,10 +405,19 @@ ORDER BY ORDINAL_POSITION};
 	my $pos = 0;
 	while (my $row = $sth->fetch)
 	{
-		if ($row->[1] eq 'enum') {
-			$row->[1] = $row->[-2];
+		next if (!$self->is_in_struct($row->[8], $row->[0]));
+		$row->[4] =~ s/^_[^']+\\'(.*)\\'/'$1'/; # fix collation on string
+		if ($row->[1] eq 'enum')
+		{
+			if ($self->{db_version} < '5.7.0') {
+				$row->[1] = $row->[-1];
+			} else {
+				$row->[1] = $row->[-2];
+			}
 		}
-		if ($row->[13] =~ /unsigned/) {
+		if ($row->[13] =~ s/(decimal.*)\s+unsigned//) {
+			$row->[1] = $1;
+		} elsif ($row->[13] =~ /unsigned/) {
 			$row->[1] .= ' unsigned';
 		}
 
@@ -505,6 +515,8 @@ sub _get_indexes
 		#Null : Contains YES if the column may contain NULL values and '' if not.
 		#Index_type : The index method used (BTREE, FULLTEXT, HASH, RTREE).
 		#Comment : Information about the index not described in its own column, such as disabled if the index is disabled. 
+			next if (!$self->is_in_struct($row->[0], $row->[4]));
+
 			my $idxname = $row->[2];
 			$row->[1] = 'UNIQUE' if (!$row->[1]);
 			$unique{$row->[0]}{$idxname} = $row->[1];
@@ -616,6 +628,8 @@ sub _foreign_key
 		if ($self->{schema} && (lc($r->[7]) ne lc($self->{schema}))) {
 			print STDERR "WARNING: Foreign key $r->[2].$r->[0] point to an other database: $r->[7].$r->[3].$r->[4], please fix it.\n";
 		}
+		next if (!$self->is_in_struct($r->[2], $r->[0]));
+		next if (!$self->is_in_struct($r->[3], $r->[4]));
 		push(@{$link{$r->[2]}{$key_name}{local}}, $r->[0]);
 		push(@{$link{$r->[2]}{$key_name}{remote}{$r->[3]}}, $r->[4]);
 		$r->[8] = 'SIMPLE'; # See pathetical documentation of mysql
@@ -666,8 +680,10 @@ sub _get_views
 
 	my %ordered_view = ();
 	my %data = ();
-	while (my $row = $sth->fetch) {
+	while (my $row = $sth->fetch)
+	{
 		$row->[1] =~ s/`$self->{schema}`\.//g;
+		$row->[1] =~ s/ AS `([^`]+\([^`]+)`//ig;
 		$row->[1] =~ s/`([^\s`,]+)`/$1/g;
 		$row->[1] =~ s/"/'/g;
 		$row->[1] =~ s/`/"/g;
@@ -830,7 +846,7 @@ sub _check_constraint
 	while (my $row = $sth->fetch)
 	{
 		# Pour chaque retour SHOW CREATE TABLE xxxx;
-		my $sql2 = "SHOW CREATE TABLE $row->[1];";
+		my $sql2 = "SHOW CREATE TABLE `$row->[1]`;";
 		my $sth2 = $self->{dbh}->prepare($sql2) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth2->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		# Parsing de   CONSTRAINT `CHK_CONSTR` CHECK (((`Age` >= 18) and (`City` = _utf8mb4'Bangalore')))
@@ -925,7 +941,7 @@ sub _get_functions
 	{
 		my $kind = $row->[7]; # FUNCTION or PROCEDURE
 		next if ( ($kind ne $self->{type}) && ($self->{type} ne 'SHOW_REPORT') );
-		my $sth2 = $self->{dbh}->prepare("SHOW CREATE $kind $row->[0]") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+		my $sth2 = $self->{dbh}->prepare("SHOW CREATE $kind `$row->[0]`") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth2->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		while (my $r = $sth2->fetch)
 		{
@@ -951,7 +967,8 @@ sub _lookup_function
 {
 	my ($self, $code, $fctname) = @_;
 
-	my $type = lc($self->{type}) . 's';
+	my $type = 'functions';
+	$type = lc($self->{type}) . 's' if ($self->{type} eq 'FUNCTION' or $self->{type} eq 'PROCEDURE');
 
 	# Replace all double quote with single quote
 	$code =~ s/"/'/g;
@@ -972,27 +989,29 @@ sub _lookup_function
 			$fct_detail{code} = "BEGIN\n    $1;\nEND\n";
 		}
 	}
-	return if (!$fct_detail{code});
 
 	# Remove any label that was before the main BEGIN block
 	$fct_detail{declare} =~ s/\s+[^\s\:]+:\s*$//gs;
 
         @{$fct_detail{param_types}} = ();
 
-        if ( ($fct_detail{declare} =~ s/(.*?)\b(FUNCTION|PROCEDURE)\s+([^\s\(]+)\s*(\(.*\))\s+RETURNS\s+(.*)//is) ||
-			($fct_detail{declare} =~ s/(.*?)\b(FUNCTION|PROCEDURE)\s+([^\s\(]+)\s*(\(.*\))//is) )
+        if ( ($fct_detail{declare} =~ s/(.*?)\b(FUNCTION|PROCEDURE)\s+([^\(]+)\s*(\(.*\))\s+RETURNS\s+(.*)//is) ||
+		($fct_detail{declare} =~ s/(.*?)\b(FUNCTION|PROCEDURE)\s+([^\(]+)\s*(\(.*\))\s*(.*)//is) )
 	{
                 $fct_detail{before} = $1;
                 $fct_detail{type} = uc($2);
                 $fct_detail{name} = $3;
                 $fct_detail{args} = $4;
 		my $tmp_returned = $5;
+		$type = lc($fct_detail{type} . 's');
 		chomp($tmp_returned);
 		if ($tmp_returned =~ s/\b(DECLARE\b.*)//is) {
 			$fct_detail{code} = $1 . $fct_detail{code};
 		}
 		if ($fct_detail{declare} =~ s/\s*COMMENT\s+(\?TEXTVALUE\d+\?|'[^\']+')//) {
 			$fct_detail{comment} = $1;
+		} elsif ($tmp_returned =~ s/\s*COMMENT\s+(\?TEXTVALUE\d+\?|'[^\']+')//) {
+			$fct_detail{comment} = $1;
 		}
 		$fct_detail{immutable} = 1 if ($fct_detail{declare} =~ s/\s*\bDETERMINISTIC\b//is);
 		$fct_detail{before} = ''; # There is only garbage for the moment
@@ -1018,6 +1037,11 @@ sub _lookup_function
 		$fct_detail{immutable} = 1 if ($self->{$type}{$fctname}{immutable} eq 'YES');
 		$fct_detail{security} = $self->{$type}{$fctname}{security};
 
+		if (!$fct_detail{code} && $tmp_returned) {
+			$tmp_returned .= ';' if ($tmp_returned !~ /;$/s);
+			$fct_detail{code} = "\n$tmp_returned\nEND;"
+		}
+
 		# Procedure that have out parameters are functions with PG
 		if ($type eq 'procedures' && $fct_detail{args} =~ /\b(OUT|INOUT)\b/) {
 			# set return type to empty to avoid returning void later
@@ -1033,10 +1057,10 @@ sub _lookup_function
 		}
 		# Now convert types
 		if ($fct_detail{args}) {
-			$fct_detail{args} = replace_sql_type($fct_detail{args}, $self->{pg_numeric_type}, $self->{default_numeric}, $self->{pg_integer_type}, %{ $self->{data_type} });
+			$fct_detail{args} = replace_sql_type($self, $fct_detail{args});
 		}
 		if ($fct_detail{declare}) {
-			$fct_detail{declare} = replace_sql_type($fct_detail{declare}, $self->{pg_numeric_type}, $self->{default_numeric}, $self->{pg_integer_type}, %{ $self->{data_type} });
+			$fct_detail{declare} = replace_sql_type($self, $fct_detail{declare});
 		}
 
 		$fct_detail{args} =~ s/\s+/ /gs;
@@ -1155,7 +1179,7 @@ sub replace_mysql_variables
 	return ($code, $declare);
 }
 
-sub _list_all_funtions
+sub _list_all_functions
 {
 	my $self = shift;
 
@@ -1191,6 +1215,7 @@ sub _sql_type
         my ($self, $type, $len, $precision, $scale, $default, $no_blob_to_oid) = @_;
 
 	my $data_type = '';
+	chomp($type);
 
 	# Simplify timestamp type
 	$type =~ s/TIMESTAMP\s*\(\s*\d+\s*\)/TIMESTAMP/i;
@@ -1217,6 +1242,9 @@ sub _sql_type
         if (exists $self->{data_type}{uc($type)})
 	{
 		$type = uc($type); # Force uppercase
+
+		$len *= 2 if ($len > 0 && $self->{double_max_varchar} && $type =~ /VARCHAR/);
+
 		if ($len)
 		{
 			if ( ($type eq "CHAR") || ($type =~ /VARCHAR/) )
@@ -1236,10 +1264,14 @@ sub _sql_type
 			}
 		       	elsif ($type =~ /(TINYINT|SMALLINT|MEDIUMINT|INTEGER|BIGINT|INT|REAL|DOUBLE|FLOAT|DECIMAL|NUMERIC)/i)
 			{
+				if ($type =~ /(TINYINT|SMALLINT|MEDIUMINT|INTEGER|BIGINT|INT)/i) {
+					$scale = 0;
+					$precision = 0;
+				}
 				# This is an integer
 				if (!$scale)
 				{
-					if ($type =~ /UNSIGNED/&& $precision)
+					if ($type =~ /UNSIGNED/ && $precision)
 					{
 						# Replace MySQL type UNSIGNED in cast
 						$type =~ s/TINYINT UNSIGNED/smallint/igs;
@@ -1251,7 +1283,11 @@ sub _sql_type
 					}
 					elsif ($precision)
 					{
-						if ($self->{pg_integer_type})
+						if ($type =~ /(REAL|DOUBLE|FLOAT)/i)
+						{
+							return $self->{data_type}{$type};
+						}
+						elsif ($self->{pg_integer_type})
 						{
 							if ($precision < 5) {
 								return 'smallint';
@@ -1298,14 +1334,14 @@ sub _sql_type
 
 sub replace_sql_type
 {
-        my ($str, $pg_numeric_type, $default_numeric, $pg_integer_type, %data_type) = @_;
+        my ($self, $str) = @_;
 
 	$str =~ s/with local time zone/with time zone/igs;
 	$str =~ s/([A-Z])ORA2PG_COMMENT/$1 ORA2PG_COMMENT/igs;
 
 	# Remove any reference to UNSIGNED AND ZEROFILL
 	# but translate CAST( ... AS unsigned) before.
-	$str =~ s/(\s+AS\s+)UNSIGNED/$1$data_type{'UNSIGNED'}/gis;
+	$str =~ s/(\s+AS\s+)UNSIGNED/$1$self->{data_type}{'UNSIGNED'}/gis;
 	$str =~ s/\b(UNSIGNED|ZEROFILL)\b//gis;
 
 	# Remove BINARY from CHAR(n) BINARY and VARCHAR(n) BINARY
@@ -1314,7 +1350,7 @@ sub replace_sql_type
 
 	# Replace type with precision
 	my $mysqltype_regex = '';
-	foreach (keys %data_type) {
+	foreach (keys %{$self->{data_type}}) {
 		$mysqltype_regex .= quotemeta($_) . '|';
 	}
 	$mysqltype_regex =~ s/\|$//;
@@ -1330,8 +1366,8 @@ sub replace_sql_type
 			$str =~ s/\)/\%\|\%/s;
 			next;
 		}
-		if (exists $data_type{"$type($args)"}) {
-			$str =~ s/\b$type\($args\)/$data_type{"$type($args)"}/igs;
+		if (exists $self->{data_type}{"$type($args)"}) {
+			$str =~ s/\b$type\($args\)/$self->{data_type}{"$type($args)"}/igs;
 			next;
 		}
 		if ($backstr =~ /_$/) {
@@ -1347,11 +1383,11 @@ sub replace_sql_type
 			# Type CHAR have default length set to 1
 			# Type VARCHAR must have a specified length
 			$len = 1 if (!$len && ($type eq "CHAR"));
-			$str =~ s/\b$type\b\s*\([^\)]+\)/$data_type{$type}\%\|$len\%\|\%/is;
+			$str =~ s/\b$type\b\s*\([^\)]+\)/$self->{data_type}{$type}\%\|$len\%\|\%/is;
 		} elsif ($precision && ($type =~ /(BIT|TINYINT|SMALLINT|MEDIUMINT|INTEGER|BIGINT|INT|REAL|DOUBLE|FLOAT|DECIMAL|NUMERIC)/)) {
 			if (!$scale) {
 				if ($type =~ /(BIT|TINYINT|SMALLINT|MEDIUMINT|INTEGER|BIGINT|INT)/) {
-					if ($pg_integer_type) {
+					if ($self->{pg_integer_type}) {
 						if ($precision < 5) {
 							$str =~ s/\b$type\b\s*\([^\)]+\)/smallint/is;
 						} elsif ($precision <= 9) {
@@ -1363,13 +1399,13 @@ sub replace_sql_type
 						$str =~ s/\b$type\b\s*\([^\)]+\)/numeric\%\|$precision\%\|\%/i;
 					}
 				} else {
-					$str =~ s/\b$type\b\s*\([^\)]+\)/$data_type{$type}\%\|$precision\%\|\%/is;
+					$str =~ s/\b$type\b\s*\([^\)]+\)/$self->{data_type}{$type}\%\|$precision\%\|\%/is;
 				}
 			} else {
 				if ($type =~ /DOUBLE/) {
 					$str =~ s/\b$type\b\s*\([^\)]+\)/decimal\%\|$args\%\|\%/is;
 				} else {
-					$str =~ s/\b$type\b\s*\([^\)]+\)/$data_type{$type}\%\|$args\%\|\%/is;
+					$str =~ s/\b$type\b\s*\([^\)]+\)/$self->{data_type}{$type}\%\|$args\%\|\%/is;
 				}
 			}
 		} else {
@@ -1384,13 +1420,13 @@ sub replace_sql_type
 	# Replace datatype even without precision
 	my %recover_type = ();
 	my $i = 0;
-	foreach my $type (sort { length($b) <=> length($a) } keys %data_type)
+	foreach my $type (sort { length($b) <=> length($a) } keys %{$self->{data_type}})
 	{
 		# Keep enum as declared, we are not in table definition
 		next if (uc($type) eq 'ENUM');
 		while ($str =~ s/\b$type\b/%%RECOVER_TYPE$i%%/is)
 		{
-			$recover_type{$i} = $data_type{$type};
+			$recover_type{$i} = $self->{data_type}{$type};
 			$i++;
 		}
 	}
@@ -1494,7 +1530,36 @@ WHERE PARTITION_NAME IS NOT NULL
 		$row->[6] =~ s/\`//g;
 		$row->[3] =~ s/\`//g;
 
-		$row->[5] = 'HASH' if ($row->[5] =~ /KEY/);
+		# Partition by KEY can be converted to HASH partition
+		# but we need to gather the PK or UK definition
+		my @ucol = ();
+		if ($row->[5] =~ /KEY/)
+		{
+			$row->[5] = 'HASH';
+			if ($row->[6])
+			{
+				$row->[5] = 'HASH COLUMNS';
+			}
+			else
+			{
+				my $sql = "SHOW INDEX FROM `$row->[0]`";
+				my $sth2 = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+				$sth2->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+				while (my $r = $sth2->fetch)
+				{
+					if ($r->[2] eq 'PRIMARY') {
+						push(@ucol, $r->[4]) if (!grep(/^$r->[4]$/, @ucol));
+					} elsif (!$r->[1]) {
+						push(@ucol, $r->[4]) if (!grep(/^$r->[4]$/, @ucol));
+					}
+				}
+				$sth2->finish;
+			}
+		}
+		else
+		{
+			push(@ucol, $row->[6]);
+		}
 
 		if ($row->[5] =~ s/ COLUMNS//)
 		{
@@ -1505,6 +1570,12 @@ WHERE PARTITION_NAME IS NOT NULL
 				$i++;
 			}
 		}
+		elsif ($row->[5] eq 'HASH')
+		{
+			for (my $i = 0; $i <= $#ucol;$i++) {
+				push(@{$parts{$row->[0]}{$row->[1]}{info}}, { 'type' => $row->[5], 'value' => $row->[3], 'column' => $ucol[$i], 'colpos' => $i, 'tablespace' => $row->[4], 'owner' => '' });
+			}
+		}
 		else
 		{
 			@{$parts{$row->[0]}{$row->[1]}{info}} = ( { 'type' => $row->[5], 'value' => $row->[3], 'expression' => $row->[6], 'colpos' => 0, 'tablespace' => $row->[4], 'owner' => '' } );
@@ -1564,14 +1635,14 @@ WHERE SUBPARTITION_NAME IS NOT NULL AND SUBPARTITION_EXPRESSION IS NOT NULL
 	return \%subparts, \%default;
 }
 
-=head2 _get_partitions_type
+=head2 _get_partitions_list
 
 This function implements a MySQL-native partitions information.
 Return a hash of the partition table_name => type
 
 =cut
 
-sub _get_partitions_type
+sub _get_partitions_list
 {
 	my($self) = @_;
 
@@ -1644,50 +1715,60 @@ FROM INFORMATION_SCHEMA.PARTITIONS WHERE PARTITION_NAME IS NOT NULL
 		if ($row->[1] =~ /KEY/)
 		{
 			$parts{"\L$row->[0]\E"}{type} = 'HASH';
-			my $sql = "SHOW INDEX FROM `$row->[0]`";
-			my $sth2 = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-			$sth2->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-			my @ucol = ();
-			while (my $r = $sth2->fetch)
+
+			if ($row->[6])
 			{
-				#Table : The name of the table.
-				#Non_unique : 0 if the index cannot contain duplicates, 1 if it can.
-				#Key_name : The name of the index. If the index is the primary key, the name is always PRIMARY.
-				#Seq_in_index : The column sequence number in the index, starting with 1.
-				#Column_name : The column name.
-				#Collation : How the column is sorted in the index. In MySQL, this can have values “A” (Ascending) or NULL (Not sorted).
-				#Cardinality : An estimate of the number of unique values in the index.
-				#Sub_part : The number of indexed characters if the column is only partly indexed, NULL if the entire column is indexed.
-				#Packed : Indicates how the key is packed. NULL if it is not.
-				#Null : Contains YES if the column may contain NULL values and '' if not.
-				#Index_type : The index method used (BTREE, FULLTEXT, HASH, RTREE).
-				#Comment : Information about the index not described in its own column, such as disabled if the index is disabled. 
-				if ($r->[2] eq 'PRIMARY') {
-					push(@{ $parts{"\L$row->[0]\E"}{columns} }, $r->[4]) if (!grep(/^$r->[4]$/, @{ $parts{"\L$row->[0]\E"}{columns} }));
-				} elsif (!$r->[1]) {
-					push(@ucol, $r->[4]) if (!grep(/^$r->[4]$/, @ucol));
-				}
+				$parts{"\L$row->[0]\E"}{type} = 'HASH COLUMNS';
 			}
-			$sth2->finish;
-			if ($#{ $parts{"\L$row->[0]\E"}{columns} } < 0) {
-				if ($#ucol >= 0) {
-					push(@{ $parts{"\L$row->[0]\E"}{columns} }, @ucol);
-				} else {
-					$row->[6] =~ s/[\(\)\s]//g;
-					@{ $parts{"\L$row->[0]\E"}{columns} } = split(',', $row->[6]);
+			else
+			{
+				my $sql = "SHOW INDEX FROM `$row->[0]`";
+				my $sth2 = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+				$sth2->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+				my @ucol = ();
+				while (my $r = $sth2->fetch)
+				{
+					#Table : The name of the table.
+					#Non_unique : 0 if the index cannot contain duplicates, 1 if it can.
+					#Key_name : The name of the index. If the index is the primary key, the name is always PRIMARY.
+					#Seq_in_index : The column sequence number in the index, starting with 1.
+					#Column_name : The column name.
+					#Collation : How the column is sorted in the index. In MySQL, this can have values “A” (Ascending) or NULL (Not sorted).
+					#Cardinality : An estimate of the number of unique values in the index.
+					#Sub_part : The number of indexed characters if the column is only partly indexed, NULL if the entire column is indexed.
+					#Packed : Indicates how the key is packed. NULL if it is not.
+					#Null : Contains YES if the column may contain NULL values and '' if not.
+					#Index_type : The index method used (BTREE, FULLTEXT, HASH, RTREE).
+					#Comment : Information about the index not described in its own column, such as disabled if the index is disabled. 
+					if ($r->[2] eq 'PRIMARY') {
+						push(@{ $parts{"\L$row->[0]\E"}{columns} }, $r->[4]) if (!grep(/^$r->[4]$/, @{ $parts{"\L$row->[0]\E"}{columns} }));
+					} elsif (!$r->[1]) {
+						push(@ucol, $r->[4]) if (!grep(/^$r->[4]$/, @ucol));
+					}
+				}
+				$sth2->finish;
+				if ($#{ $parts{"\L$row->[0]\E"}{columns} } < 0) {
+					if ($#ucol >= 0) {
+						push(@{ $parts{"\L$row->[0]\E"}{columns} }, @ucol);
+					} else {
+						$row->[6] =~ s/[\(\)\s]//g;
+						@{ $parts{"\L$row->[0]\E"}{columns} } = split(',', $row->[6]);
+					}
 				}
 			}
 
 		}
+
 		if ($parts{"\L$row->[0]\E"}{type} =~ s/ COLUMNS//)
 		{
 			$row->[6] =~ s/[\(\)\s]//g;
 			@{ $parts{"\L$row->[0]\E"}{columns} } = split(',', $row->[6]);
 		}
-		else
+		elsif ($row->[6])
 		{
 			$parts{"\L$row->[0]\E"}{expression} = $row->[6];
 		}
+
 	}
 	$sth->finish;
 
@@ -2072,9 +2153,6 @@ $condition
 ORDER BY ORDINAL_POSITION
 };
 
-	if ($self->{db_version} < '5.5.0') {
-		$sql =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
-	}
 	my $sth = $self->{dbh}->prepare($sql);
 	if (!$sth) {
 		$self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
@@ -2084,6 +2162,7 @@ ORDER BY ORDINAL_POSITION
 	my %data = ();
 	while (my $row = $sth->fetch)
 	{
+		$row->[2] =~ s/^_[^']+\\'(.*)\\'/'$1'/; # fix collation on string
 		$data{$row->[3]}{"$row->[0]"}{nullable} = $row->[1];
 		$data{$row->[3]}{"$row->[0]"}{default} = $row->[2];
 		# Store the data type of the column following its position
@@ -2156,36 +2235,43 @@ FROM INFORMATION_SCHEMA.PARTITIONS WHERE SUBPARTITION_NAME IS NOT NULL
 		$parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{type} = $row->[1];
 		$row->[7] =~ s/\`//g;
 
-		if ($parts{"\L$row->[0]\E"}{type} =~ /KEY/)
+		if ($parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{type} =~ /KEY/)
 		{
-			$parts{"\L$row->[0]\E"}{type} = 'HASH';
-			my $sql = "SHOW INDEX FROM `$row->[0]`";
-			my $sth2 = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-			$sth2->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-			my @ucol = ();
-			while (my $r = $sth2->fetch)
+			$parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{type} = 'HASH';
+			if ($row->[7])
 			{
-				#Table : The name of the table.
-				#Non_unique : 0 if the index cannot contain duplicates, 1 if it can.
-				#Key_name : The name of the index. If the index is the primary key, the name is always PRIMARY.
-				#Seq_in_index : The column sequence number in the index, starting with 1.
-				#Column_name : The column name.
-				#Collation : How the column is sorted in the index. In MySQL, this can have values “A” (Ascending) or NULL (Not sorted).
-				#Cardinality : An estimate of the number of unique values in the index.
-				#Sub_part : The number of indexed characters if the column is only partly indexed, NULL if the entire column is indexed.
-				#Packed : Indicates how the key is packed. NULL if it is not.
-				#Null : Contains YES if the column may contain NULL values and '' if not.
-				#Index_type : The index method used (BTREE, FULLTEXT, HASH, RTREE).
-				#Comment : Information about the index not described in its own column, such as disabled if the index is disabled. 
-				if ($r->[2] eq 'PRIMARY') {
-					push(@{ $parts{"\L$row->[0]\E"}{columns} }, $r->[4]) if (!grep(/^$r->[4]$/, @{ $parts{"\L$row->[0]\E"}{columns} }));
-				} elsif (!$row->[1]) {
-					push(@ucol, $r->[4]) if (!grep(/^$r->[4]$/, @ucol));
-				}
+				$parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{type} = 'HASH COLUMNS';
 			}
-			$sth2->finish;
-			if ($#{ $parts{"\L$row->[0]\E"}{columns} } < 0) {
-				push(@{ $parts{"\L$row->[0]\E"}{columns} }, @ucol);
+			else
+			{
+				my $sql = "SHOW INDEX FROM `$row->[0]`";
+				my $sth2 = $self->{dbh}->prepare($sql) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+				$sth2->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+				my @ucol = ();
+				while (my $r = $sth2->fetch)
+				{
+					#Table : The name of the table.
+					#Non_unique : 0 if the index cannot contain duplicates, 1 if it can.
+					#Key_name : The name of the index. If the index is the primary key, the name is always PRIMARY.
+					#Seq_in_index : The column sequence number in the index, starting with 1.
+					#Column_name : The column name.
+					#Collation : How the column is sorted in the index. In MySQL, this can have values “A” (Ascending) or NULL (Not sorted).
+					#Cardinality : An estimate of the number of unique values in the index.
+					#Sub_part : The number of indexed characters if the column is only partly indexed, NULL if the entire column is indexed.
+					#Packed : Indicates how the key is packed. NULL if it is not.
+					#Null : Contains YES if the column may contain NULL values and '' if not.
+					#Index_type : The index method used (BTREE, FULLTEXT, HASH, RTREE).
+					#Comment : Information about the index not described in its own column, such as disabled if the index is disabled. 
+					if ($r->[2] eq 'PRIMARY') {
+						push(@{ $parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{columns} }, $r->[4]) if (!grep(/^$r->[4]$/, @{ $parts{"\L$row->[0]\E"}{columns} }));
+					} elsif (!$row->[1]) {
+						push(@ucol, $r->[4]) if (!grep(/^$r->[4]$/, @ucol));
+					}
+				}
+				$sth2->finish;
+				if ($#{ $parts{"\L$row->[0]\E"}{columns} } < 0) {
+					push(@{ $parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{columns} }, @ucol);
+				}
 			}
 		}
 
@@ -2194,7 +2280,7 @@ FROM INFORMATION_SCHEMA.PARTITIONS WHERE SUBPARTITION_NAME IS NOT NULL
 			$row->[7] =~ s/[\(\)\s]//g;
 			@{ $parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{columns} } = split(',', $row->[7]);
 		}
-		else
+		elsif ($row->[7])
 		{
 			$parts{"\L$row->[0]\E"}{"\L$row->[3]\E"}{expression} = $row->[7];
 		}
@@ -2291,7 +2377,7 @@ sub _get_plsql_metadata
 		push(@fct_done, "$row->[0]");
 		$self->{function_metadata}{'unknown'}{'none'}{$row->[0]}{type} = $row->[2];
 		$self->{function_metadata}{'unknown'}{'none'}{$row->[0]}{text} = $row->[3];
-		my $sth2 = $self->{dbh}->prepare("SHOW CREATE $row->[2] $row->[0]") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+		my $sth2 = $self->{dbh}->prepare("SHOW CREATE $row->[2] `$row->[0]`") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth2->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		while (my $r = $sth2->fetch) {
 			$self->{function_metadata}{'unknown'}{'none'}{$row->[0]}{text} = $r->[2];
@@ -2302,7 +2388,8 @@ sub _get_plsql_metadata
 	$sth->finish();
 
 	# Look for functions/procedures
-	foreach my $name (sort keys %{$self->{function_metadata}{'unknown'}{'none'}}) {
+	foreach my $name (sort keys %{$self->{function_metadata}{'unknown'}{'none'}})
+	{
 		# Retrieve metadata for this function after removing comments
 		$self->_remove_comments(\$self->{function_metadata}{'unknown'}{'none'}{$name}{text}, 1);
 		$self->{comment_values} = ();
@@ -2414,8 +2501,22 @@ sub _col_count
 {
         my ($self, $name) = @_;
 
-	# Not supported
-	return;
+	my $sql = qq{SELECT TABLE_SCHEMA, TABLE_NAME, count(*)
+FROM INFORMATION_SCHEMA.COLUMNS
+GROUP BY TABLE_SCHEMA, TABLE_NAME
+};
+	my $sth = $self->{dbh}->prepare($sql);
+	if (!$sth) {
+		$self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+	}
+	$sth->execute() or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
+
+	my %data = ();
+	while (my $row = $sth->fetch) {
+		$data{$row->[1]} = $row->[2];
+	}
+
+	return %data;
 }
 
 1;
